/***********************************************************************
D E S C R I P T I O N

This program uses the SAS Open Metadata API data step functions to query
the metadata repository and return a list of all SAS Libraries defined
in metadata. The results are returned to a SAS data set in the WORK 
library. This code does not write to or modify objects in the metadata
server. Included data are:
  library metadata ID
  the library name
  libref
  engine
  path on the file system, or if DBMS data, the DBMS path
  server contexts to which the library is associated
  whether the library is marked pre-assigned, or not
  
The server context is a single character variable containing the server
context names within quotes and delimited by a space.  

C O M P A T I B I L I T Y

SAS 9.1.3, 9.2, 9.3, 9.4

E D I T S

This program will require editing in order to run on your system, some
of which are listed here:

Modify the Meta* options to supply correct connection parameters to the
metadata server that contains the user information.

Modify the reporting code to generate the results in a form useful to
you. This sample demonstrates using PROC PRINT.

***********************************************************************/


/**  E D I T  H E R E  **/
options
  metaserver="sasbi.demo.sas.com"
  metaport=8561
  metauser="SASBI\sasadm"
  metapass="Orion123"
  metarepository="Foundation"
  metaprotocol=BRIDGE;

/**  E N D  E D I T  **/
  

/* Create a data set containing each Library Name
   defined in metadata. */

data work.Libraries;
  length
LibId
LibName $ 32
LibRef
LibEngine $ 8
LibPath $ 256
ServerContext
uri
uri2
type $ 256
server $ 32
mdCreated
mdUpdated $ 18
IsPreassigned $ 8
id $ 20 
;

  label
LibId = "Library Id"
LibName = "Library Name"
LibRef = "SAS Libref"
LibEngine = "Library Engine"
ServerContext = "Server Contexts"
LibPath = "Library Path"
mdCreated = "MetadataCreated"
mdUpdated = "MetadataUpdated"
IsPreassigned = "Library Is Preassigned"
;

/* initialize values */
  call missing(LibId,LibName,LibRef,LibEngine,LibPath,
       ServerContext,uri,uri2,type,server,mdCreated,mdUpdated,
       IsPreassigned);
  call missing(Id);
  n=1;
  n2=1;

  /* get each Library object */
  rc=metadata_getnobj("omsobj:SASLibrary?@Id contains '.'",n,uri);
  if rc<=0 then put "NOTE: rc=" rc
    "There are no Libraries defined in this repository"
    " or there was an error reading the repository.";
  do while(rc>0);
     objrc=metadata_getattr(uri,"Id",LibId);
     objrc=metadata_getattr(uri,"Name",LibName);
	 objrc=metadata_getattr(uri,"Libref",LibRef);
	 objrc=metadata_getattr(uri,"Engine",LibEngine);
	 objrc=metadata_getattr(uri,"MetadataCreated",mdCreated);
	 objrc=metadata_getattr(uri,"MetadataUpdated",mdUpdated);
	 objrc=metadata_getattr(uri,"IsPreassigned",IsPreassigned);

	 /* get association containing server context */
	 objrc=metadata_getnasn(uri,"DeployedComponents",n2,uri2);
	 if objrc<=0 then
       do;
         put "NOTE: There is no DeployedComponents association for "
             LibName +(-1)", and therefore no Server Context.";
	     ServerContext="";
	   end;
	   do while(objrc>0);
         objrc=metadata_getattr(uri2,"Name",server);
	     if n2=1 then ServerContext=quote(trim(server));
		 else ServerContext=trim(ServerContext)||" "||quote(trim(server));
	     n2+1;
         objrc=metadata_getnasn(uri,"DeployedComponents",n2,uri2);
	   end; /*do while objrc*/

     n2=1;

	 /* get association containing path */
	 objrc=metadata_getnasn(uri,"UsingPackages",n2,uri2);
	 if objrc<=0 then
       do;
         put "NOTE: There is no UsingPackages association for "
             LibName +(-1)", and therefore no Path.";
	     LibPath="";
	   end;
	   do while(objrc>0);
	     objrc=metadata_resolve(uri2,type,id);
		 if type='Directory' then objrc=metadata_getattr(uri2,"DirectoryName",LibPath);
		 else if type='DatabaseSchema' then objrc=metadata_getattr(uri2, "Name", LibPath);
		 else LibPath="*unknown*";
	     output;
	     LibPath="";
         n2+1;
	     objrc=metadata_getnasn(uri,"UsingPackages",n2,uri2);
	   end; /*do while objrc*/

     ServerContext="";
     n+1;
	 n2=1;
     rc=metadata_getnobj("omsobj:SASLibrary?@Id contains '.'",n,uri);

  end; /*do while rc*/

  keep
LibId
LibName
LibRef
LibEngine
ServerContext
LibPath
mdCreated
mdUpdated
IsPreassigned
;
run;

/*****
   Write basic listing of data
   Change DATA= to same data set name you specified above.
   Use a WHERE clause to subset the output to a specific
   Server Context, such as:

       *where indexw(ServerContext,'"SASApp"') > 0;

*****/
proc print data=work.Libraries;
run;
